class TrackerChangelogEvents:
    ISSUE_WORKFLOW = "IssueWorkflow"


class TrackerWorkflowTypes:
    TRANSITION = "status"
    RESOLVE_ISSUE = "resolution"


class YandexTrackerLanguages:
    RU = "ru"
    EN = "en"


class TimeDeltaOut:
    SECONDS = "seconds"
    MINUTES = "minutes"

class ClickhouseProto:
    HTTPS = "https"
    HTTP = "http"
